/*
 * RenderArea.h
 * 
 * Copyright (C) 2008 Matthias Schiffer <matthias@gamezock.de>
 * 
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ZOOMEDIT_GUI_RENDERAREA_H_
#define ZOOMEDIT_GUI_RENDERAREA_H_

#include <gtkmm/drawingarea.h>
#include <gtkmm/scrollbar.h>
#include <libglademm/xml.h>
#include <gtk/gtkgl.h>

namespace ZoomEdit {

namespace View {
class View;
}

namespace Gui {

class RenderArea : public Gtk::DrawingArea {
  public:
    RenderArea(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml> &xml);
    
    View::View* getView() const {return view;}
    void setView(View::View *view0) {
      view = view0;
      queue_draw();
    }
    
    float getViewWidth() const {
      return get_width()/scale;
    }
    
    float getViewHeight() const {
      return get_height()/scale;
    }
    
    float getImageWidth() const {return 10;}
    float getImageHeight() const {return 10;}
    
    float getScale() const {return scale;}
    
    float getXCenter() const {return xCenter;}
    float getYCenter() const {return yCenter;}
    
  private:
    static GdkGLConfig *glconfig;
    
    Gtk::HScrollbar *hScrollbar;
    Gtk::VScrollbar *vScrollbar;
    
    View::View *view;
    
    float xCenter, yCenter;
    
    int zoomLevel;
    float scale;
    
    void onRealize();
    bool onConfigureEvent(GdkEventConfigure*);
    bool onExposeEvent(GdkEventExpose*);
    bool onScrollEvent(GdkEventScroll *event);
    
    void zoom(int zoom, float x = 0.5f, float y = 0.5f);
    
    void updateViewport();
    void updateScrollbars(float x = 0.5f, float y = 0.5f);
    void updateScrolling();
    
    bool gdkGLBegin() {
      GtkWidget *widget = GTK_WIDGET(gobj());
      
      return gdk_gl_drawable_gl_begin(gtk_widget_get_gl_drawable(widget), gtk_widget_get_gl_context(widget));
    }
    
    void gdkSwapBuffers() {
      gdk_gl_drawable_swap_buffers(gtk_widget_get_gl_drawable(GTK_WIDGET(gobj())));
    }
    
    void gdkGLEnd() {
      gdk_gl_drawable_gl_end(gtk_widget_get_gl_drawable(GTK_WIDGET(gobj())));
    }
};

}
}

#endif /*ZOOMEDIT_GUI_RENDERAREA_H_*/
